/* license: GNU GPL 2+ (see ../COPYING) */
#include <stdio.h>
#include <stdlib.h>
#include "l_macro.h"

#define	USAGE	fprintf(stderr, "usage: convert old new\n"), exit(1)

typedef struct
{
	int		x,			/* Zeiger auf Koordinaten */
			y;
	char	s;			/* Zeiger auf Groesse */
} DotOld;

typedef struct
{
	int		x1,			/* Zeiger auf Koordinaten */
			y1,
			x2,
			y2;
	char	d;			/* Richtung */
	char	s;			/* Groesse  */
} LineOld;

typedef struct
{
	int		x1,			/* Ecken */
			y1,
			x2,
			y2;
} AreaOld;

LineTyp	l;
LineOld	ol;
AreaTyp	a;
AreaOld	oa;
DotTyp	d;
DotOld	od;

Header	head;

void	transform(FILE *fp1, FILE *fp2);

/******************************************

		Files transformieren

******************************************/
void transform(rfp, wfp)
FILE	*rfp, *wfp;
{
	static	int	magic;
			int	i, j, dn, ln, an;

	if (fread(&magic, sizeof(int), 1, rfp) != 1 || fread(&head, sizeof(Header), 1, rfp) != 1)
	{
		fprintf(stderr,"head read error\n");
		return;
	}
	if (magic != MAGIC)
	{
		fprintf(stderr, "wrong file format\n");
		return;
	}
	if (fwrite(&magic, sizeof(int), 1, wfp) != 1 || fwrite(&head, sizeof(Header), 1, wfp) != 1)
	{
		fprintf(stderr,"head write error\n");
		return;
	}
	if (fread(&dn, sizeof(int), 1, rfp) != 1)
	{
		fprintf(stderr,"dothead read error\n");
		return;
	}
	if (fwrite(&dn, sizeof(int), 1, wfp) != 1)
	{
		fprintf(stderr,"dothead write error\n");
		return;
	}
	printf("%i dots\n", dn);
	for (i = 0; i < dn; i++)
	{
		if (fread(&od, sizeof(DotOld), 1, rfp) != 1)
		{
			fprintf(stderr,"dot read error\n");
			return;
		}
		d.x = od.x;
		d.y = od.y;
		d.s = od.s;
		d.used = 0;
		if (fwrite(&d, sizeof(DotTyp), 1, wfp) != 1)
		{
			fprintf(stderr,"dot write error\n");
			return;
		}
	}
	for (j = 0; j < MAXLAYER; j++)
	{
		if (fread(&ln, sizeof(int), 1, rfp) != 1)
		{
			fprintf(stderr,"layerhead read error\n");
			return;
		}
		if (fwrite(&ln, sizeof(int), 1, wfp) != 1)
		{
			fprintf(stderr,"layerhead write error\n");
			return;
		}
		if (fread(&an, sizeof(int), 1, rfp) != 1)
		{
			fprintf(stderr,"areahead read error\n");
			return;
		}
		if (fwrite(&an, sizeof(int), 1, wfp) != 1)
		{
			fprintf(stderr,"areahead write error\n");
			return;
		}
		printf("layer %i:  %i tracks, %i areas\n", j, ln, an);
		for (i = 0; i < ln; i++)
		{
			if (fread(&ol, sizeof(LineOld), 1, rfp) != 1)
			{
				fprintf(stderr,"layer read error\n");
				return;
			}
			l.x1 = ol.x1;
			l.x2 = ol.x2;
			l.y1 = ol.y1;
			l.y2 = ol.y2;
			l.d = ol.d;
			l.s = ol.s;
			l.used = 0;
			if (fwrite(&l, sizeof(LineTyp), 1, wfp) != 1)
			{
				fprintf(stderr,"layer write error\n");
				return;
			}
		}
		for (i = 0; i < an; i++)
		{
			if (fread(&oa, sizeof(AreaOld), 1, rfp) != 1)
			{
				fprintf(stderr,"area read error\n");
				return;
			}
			a.x1 = oa.x1;
			a.x2 = oa.x2;
			a.y1 = oa.y1;
			a.y2 = oa.y2;
			a.used = 0;
			if (fwrite(&a, sizeof(AreaTyp), 1, wfp) != 1)
			{
				fprintf(stderr,"area write error\n");
				return;
			}
		}
	}
}

int main(argc, argv)
int		argc;
char	*argv[];
{
	FILE	*fp1, *fp2;

	if (argc != 3) USAGE;
	if ((fp1 = fopen(argv[1], "rb")) == NULL)
		fprintf(stderr, "can't open file: %s\n", argv[1]), exit(1);
	if ((fp2 = fopen(argv[2], "wb")) == NULL)
		fprintf(stderr, "can't open file: %s\n", argv[2]), exit(1);
	transform(fp1, fp2);
	fclose(fp1);
	fclose(fp2);
	return(0);
}
