/* license: GNU GPL 2+ (see ../COPYING) */
#include <stdio.h>
#include <stdlib.h>
#include "l_macro.h"

#define	USAGE	fprintf(stderr, "usage: convert old new\n"), exit(1)

typedef struct
{
	int		x,			/* Zeiger auf Koordinaten */
			y;
	int		s:6;		/* Zeiger auf Groesse */
	int		used:12;	/* Flag fuer "schon durchsucht" */
} OldDotTyp;

LineTyp		l;
AreaTyp		a;
DotTyp		d;
OldDotTyp	od;
TextTyp		t;

Header	head;

void	transform(FILE *fp1, FILE *fp2);

/******************************************

		Files transformieren

******************************************/
void transform(rfp, wfp)
FILE	*rfp, *wfp;
{
	static	int	magic;
			int	i, j, dn, ln, an, tn, en;

	en = 0;
	if (fread(&magic, sizeof(int), 1, rfp) != 1 || fread(&head, sizeof(Header), 1, rfp) != 1)
	{
		fprintf(stderr,"head read error\n");
		return;
	}
	if (magic != MAGIC)
	{
		fprintf(stderr, "wrong file format\n");
		return;
	}
	if (fwrite(&magic, sizeof(int), 1, wfp) != 1 || fwrite(&head, sizeof(Header), 1, wfp) != 1)
	{
		fprintf(stderr,"head write error\n");
		return;
	}
	if (fread(&dn, sizeof(int), 1, rfp) != 1)
	{
		fprintf(stderr,"dothead read error\n");
		return;
	}
	if (fwrite(&dn, sizeof(int), 1, wfp) != 1)
	{
		fprintf(stderr,"dothead write error\n");
		return;
	}
	printf("%i dots\n", dn);
	for (i = 0; i < dn; i++)
	{
		if (fread(&od, sizeof(OldDotTyp), 1, rfp) != 1)
		{
			fprintf(stderr,"dot read error\n");
			return;
		}
		d.x = od.x;
		d.y = od.y;
		d.used = od.used;
		d.s = od.s;
		d.name[0] = '\0';
		if (fwrite(&d, sizeof(DotTyp), 1, wfp) != 1)
		{
			fprintf(stderr,"dot write error\n");
			return;
		}
	}
	if (fwrite(&en, sizeof(int), 1, wfp) != 1)
	{
		fprintf(stderr,"elementhead write error\n");
		return;
	}
	for (j = 0; j < MAXLAYER; j++)
	{
		if (fread(&ln, sizeof(int), 1, rfp) != 1)
		{
			fprintf(stderr,"layerhead read error\n");
			return;
		}
		if (fwrite(&ln, sizeof(int), 1, wfp) != 1)
		{
			fprintf(stderr,"layerhead write error\n");
			return;
		}
		if (fread(&an, sizeof(int), 1, rfp) != 1)
		{
			fprintf(stderr,"areahead read error\n");
			return;
		}
		if (fread(&tn, sizeof(int), 1, rfp) != 1)
		{
			fprintf(stderr,"texthead read error\n");
			return;
		}
		if (fwrite(&an, sizeof(int), 1, wfp) != 1)
		{
			fprintf(stderr,"areahead write error\n");
			return;
		}
		if (fwrite(&tn, sizeof(int), 1, wfp) != 1)
		{
			fprintf(stderr,"texthead write error\n");
			return;
		}
		printf("layer %i:  %i tracks, %i areas, %i strings\n", j, ln, an, tn);
		for (i = 0; i < ln; i++)
		{
			if (fread(&l, sizeof(LineTyp), 1, rfp) != 1)
			{
				fprintf(stderr,"layer read error\n");
				return;
			}
			if (fwrite(&l, sizeof(LineTyp), 1, wfp) != 1)
			{
				fprintf(stderr,"layer write error\n");
				return;
			}
		}
		for (i = 0; i < an; i++)
		{
			if (fread(&a, sizeof(AreaTyp), 1, rfp) != 1)
			{
				fprintf(stderr,"area read error\n");
				return;
			}
			if (fwrite(&a, sizeof(AreaTyp), 1, wfp) != 1)
			{
				fprintf(stderr,"area write error\n");
				return;
			}
		}
		for (i = 0; i < tn; i++)
		{
			if (fread(&t, sizeof(TextTyp), 1, rfp) != 1)
			{
				fprintf(stderr,"text read error\n");
				return;
			}
			if (fwrite(&t, sizeof(TextTyp), 1, wfp) != 1)
			{
				fprintf(stderr,"area write error\n");
				return;
			}
		}
	}
}

int main(argc, argv)
int		argc;
char	*argv[];
{
	FILE	*fp1, *fp2;

	if (argc != 3) USAGE;
	if ((fp1 = fopen(argv[1], "rb")) == NULL)
		fprintf(stderr, "can't open file: %s\n", argv[1]), exit(1);
	if ((fp2 = fopen(argv[2], "wb")) == NULL)
		fprintf(stderr, "can't open file: %s\n", argv[2]), exit(1);
	transform(fp1, fp2);
	fclose(fp1);
	fclose(fp2);
	return(0);
}
