/*****************************************************

		Platinen Layout fr ATARI ST in TURBO C
					Thomas Nau (c) 1990
	license: GNU GPL 2+ (see ../COPYING)

*****************************************************/

/* Standart INCLUDE files */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ext.h>
#include <tos.h>
#include <ctype.h>

#define SHELL "SH"
#define DEFSHELL "c:\bin\sh.ttp"
#define ARGC 2

#define ARGMAGIC 0x25071962L              /* Konstanten fr Longpar-bergabe */
#define IMAGIC 114-1
#define IARGV 118-1
#define IARGC 122-1

#define L_COM

#include "l_incl.h"     /* alle anderen INCLUDE Files */

extern char **environ;

static char *argv[]={"sh",0L,0L};

/****************************************************

		3 Routinen fuer Wildcard match

****************************************************/
char *cclass(p, sub)
unsigned char	*p, sub;
{
	register	int				not, found;
	register	unsigned char	c,d;

	if ((not = (*p =='!')) != 0)
		p++;
	found=not;
	do
	{
		if (!*p)
			return(NULL);
		c=*p;
		if (p[1]=='-' && p[2] != ']')
			d = p[2], p++;
		else
			d=c;
		if (c==sub || c<=sub && sub<=d)
			found=!not;
	} while (*++p != ']');
	return(found ? (char *) (p+1) : NULL);
}

int gmatch(s, p)
char	*s, *p;
{
	register	char	sc,pc;

	if (s==NULL || p==NULL )
		return(FALSE);
	while ((pc = *p++) != '\0')
	{
		sc=*s++;
		switch (pc)
		{
			case '[':	if ((p=cclass((unsigned char *) p, (unsigned char) sc))==NULL)
							return(FALSE);
						break;
			case '?':	if (sc==0)
							return(FALSE);
						break;
			case '*':	if (!*p)
							return(TRUE);
						s--;
						while (*s)
						if (gmatch(s++,p))
							return(TRUE);
						return(FALSE);
			default:	if (sc!=pc) 
							return(FALSE);
		}
	}
	return(*s == 0);
}

int match(s, p)
char	*s, *p;
{
	return(*s != '.' && gmatch(strlwr(s) ,p));
}

/****************************************************

		ueberprueft ob Files vorhanden oder doppelt

****************************************************/
char *count_file(p, res)
char	*p;
int		*res;
{
	static	char	first[130];
			char	path[130],
					*ptr, *ptr1;
	struct	ffblk	buf;
			int		last, nr;

	if ((ptr1 = strrchr(p, '\\')) == NULL)
	{
		strcpy(path, ".");
		ptr1 = p;
	}
	else
	{
		*ptr1 = '\0';
		strcpy(path, p);
		++ptr1;
	}
	strcpy(first, path);
#ifdef ATARI
	strcat(first, "\\");
	ptr = strrchr(first, '\\') +1;
#else
	strcat(first, "/");
	ptr = strrchr(first, '/') +1;
#endif
	strcat(path, "\\*.*");
	nr = 0;
	last = findfirst(path, &buf, 0);
	while (!last)
	{
		if (match(buf.ff_name, ptr1))
		{
			if (!nr)
				strcpy(ptr, buf.ff_name);
			nr++;
		}
		last = findnext(&buf);
	}
	if (nr == 1)
		*res = FALSE;
	else
		*res = (nr == 0) ? -1 : 1;
	return(first);
}

/****************************************************

		oeffnet Library

****************************************************/
int op_ar(libenv, libfile)
char *libenv, *libfile;
{
	unsigned	int		magic;
				char	*an;

	if ((an = getenv(libenv)) == NULL)
		an = libfile;
	if ((ar_fp = fopen(an, "rb")) == NULL)
		return(TRUE);
	fseek(ar_fp, 0L, SEEK_END);
	arlen = ftell(ar_fp);
	fseek(ar_fp, 0L, SEEK_SET);
	if ((fread(&magic, sizeof(unsigned int), 1, ar_fp) != 1) || (magic != LIBMAGIC))
	{
		cl_ar();
		return(TRUE);
	}
	arlen -= 2;
	return(FALSE);
}

/****************************************************

		schliesst Library

****************************************************/
void cl_ar()
{
	fclose(ar_fp);
}

/****************************************************

		ueberspringt n bytes

****************************************************/
void arskip(n)
unsigned long n;
{
	fseek(ar_fp, n, SEEK_CUR);
	arlen -= n;
}

/****************************************************

		liest Lib header

****************************************************/
int r_ar_head()
{
	if (fread(&libhead, sizeof(LibHead), 1, ar_fp) != 1)
	{
		cl_ar();
		return(TRUE);
	}
	arlen -= sizeof(LibHead);
	return(FALSE);
}

/****************************************************

		liefert Zeiger auf Modulnamen in libhead

****************************************************/
char *objname(lptr)
LibHead	*lptr;
{
	static	char	s[NAMELEN+1];

	strncpy(s, lptr->name, NAMELEN);
	s[NAMELEN] = '\0';
	return(s);
}

/****************************************************

		setzt keypointer

****************************************************/
void set_kp()
{
	register	int		j, found;
				int		magic;
				char	s[NAMELEN+1];
	
	while (arlen >= sizeof(LibHead))
	{
		r_ar_head();
		found = FALSE;
		j = 0;
		while (j < 256 && !found)
		{
			sprintf(s, "key%d.lay", j);
			if (match(objname(&libhead), s) && keys[j].l == NULL)
				found = TRUE;
			else
				j++;
		}
		if (!found)
			arskip(libhead.length);
		else
		{
			if (fread(&magic, sizeof(int), 1, ar_fp) != 1 || fread(&keys[j].h, sizeof(Header), 1, ar_fp) != 1)
			{
				say_error("error reading key data header...");
				return;
			}
			if (magic != TEMAGIC)
			{
				say_error("wrong file format for key library !");
				return;
			}
			arlen -= (sizeof(Header) +sizeof(int));
			if (fread(&keys[j].n, sizeof(int), 1, ar_fp) != 1)
			{
				say_error("error reading key data length...");
				return;
			}
			arlen -= sizeof(int);
			if (keys[j].n > 0)
			{
				if ((keys[j].l = calloc(keys[j].n, sizeof(LineTyp))) == NULL)
				{
					say_error("no memory for key data left...");
					return;
				}
				if (fread(keys[j].l, sizeof(LineTyp), keys[j].n, ar_fp) != keys[j].n)
				{
					say_error("error reading key data...");
					return;
				}
				arlen -= (sizeof(LineTyp) * keys[j].n);
			}
		}
	}
}

/****************************************************

		ueberprueft ob Elemente vorhanden oder doppelt

****************************************************/
long count_ele(p, res)
char	*p;
int		*res;
{
	char	*ptr;
	long	pos;
	int		nr;

	nr = 0;
	if ((ptr = strrchr(p, '\\')) == NULL)
		ptr = p;
	else
		ptr++;
	if (op_ar("PINLIB", PINFILE))
	{
		*res = 2;
		return(0);
	}
	while (arlen >= sizeof(LibHead))
	{
		r_ar_head();
		if (match(objname(&libhead), ptr))
		{
			if (!nr)
				pos = ftell(ar_fp);
			nr++;
		}
		arskip(libhead.length);
	}
	cl_ar();	
	if (nr == 1)
		*res = FALSE;
	else
		*res = (nr == 0) ? -1 : 1;
	return(pos);
}

/****************************************************

		ueberprueft ob Module vorhanden oder doppelt

****************************************************/
long count_mod(p, res)
char	*p;
int		*res;
{
	char	*ptr;
	long	pos;
	int		nr;

	nr = 0;
	if ((ptr = strrchr(p, '\\')) == NULL)
		ptr = p;
	else
		ptr++;
	if (op_ar("LAYLIB", LIBFILE))
	{
		*res = 2;
		return(0);
	}
	while (arlen >= sizeof(LibHead))
	{
		r_ar_head();
		if (match(objname(&libhead), ptr))
		{
			if (!nr)
				pos = ftell(ar_fp);
			nr++;
		}
		arskip(libhead.length);
	}
	cl_ar();	
	if (nr == 1)
		*res = FALSE;
	else
		*res = (nr == 0) ? -1 : 1;
	return(pos);
}

/****************************************************

		liest File mit Wildcards

****************************************************/
void read_file(f)
char	*f;
{
	char	*fn;
	int		res;

	fn = count_file(f, &res);
	switch(res)
	{
		case	-1	:	say_error("no matching file"); break;
		case	0	:	load_lay(fn); 
						update_scr();
						break;
		case	1	:	say_error("too many matching files"); break;
	}
}

/****************************************************

		merged File mit Wildcards

****************************************************/
void merg_file(f)
char	*f;
{
	char	*fn;
	int		res;

	fn = count_file(f, &res);
	switch(res)
	{
		case	-1	:	say_error("no matching file"); break;
		case	0	:	merge_lay(fn); 
						update_scr();
						break;
		case	1	:	say_error("too many matching files"); break;
	}
}

/****************************************************

		merged File mit Wildcards aus Lib

****************************************************/
void merg_lib(f)
char	*f;
{
	long	pos;
	int		res, x, y;

	pos = count_mod(f, &res);
	switch(res)
	{
		case	-1	:	say_error("no matching file"); break;
		case	0	:	if (op_ar("LAYLIB", LIBFILE))
							return;
						fseek(ar_fp, pos, SEEK_SET);
						if (!read_head(ar_fp) && !mbl_head())
						{
							act_coord(&x, &y);
							if (!chk_head(x, y))
							{
								info.changed = TRUE;
								if (!merg_dots(ar_fp, x, y))
									merg_layer(ar_fp, x, y);
								strcpy(mod_name, f);
							}
						}
						cl_ar();
						update_scr();
						break;
		case	1	:	say_error("too many matching files"); break;
		case	2	:	say_error("no library file"); break;
	}
}

/****************************************************

		liest File mit Wildcards aus Lib

****************************************************/
void read_lib(f)
char	*f;
{
	long	pos;
	int		res;

	pos = count_mod(f, &res);
	switch(res)
	{
		case	-1	:	say_error("no matching file"); break;
		case	0	:	if (op_ar("LAYLIB", LIBFILE))
							return;
						fseek(ar_fp, pos, SEEK_SET);
						if (info.changed)
							if (get_ok("save old data first ?"))
								save_lay(((filename[0]) ? filename : NULL), TRUE);
						if (!read_head(ar_fp))
							if (!read_dots(ar_fp))
								if (!read_layer(ar_fp))
									info.changed = FALSE;
						cl_ar();
						update_scr();
						break;
		case	1	:	say_error("too many matching files"); break;
		case	2	:	say_error("no library file"); break;
	}
}

/****************************************************

		initialisiert keytables aus Library

****************************************************/
void init_keys()
{
	register	int	i;

	for (i = 0; i < 256; i++)
		keys[i].l = NULL;
	if (!op_ar("KEYLIB", KEYFILE))
	{
		set_kp();
		cl_ar();
	}
}

#ifdef ATARI
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/*                                                                     */
/*        system - C Bchereifunktion fr den ATRAI ST in Turbo C      */
/*                (c)   B.Dubler 17.7.1989                            */
/*                                                                     */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

int shell(command)
char *command;
{       register char *adr,*p,*q,*env=NULL,**e;
        register int r1,erg,len;
        COMMAND com;

        static char *shell;

        if (!(shell=getenv(SHELL))) shell=DEFSHELL;

        for (len=0,e=environ;*e;len+=strlen(*e++)+1);
        if ((env=malloc((size_t)(len+1)))==NULL) return -1;
        for (e=environ,p=env;*e;strcpy(p,*e),p+=strlen(*e++)+1);
        *p='\0';

        for (p=command;*p && isspace(*p);p++);
        if ((q=malloc(strlen(p)+1))==NULL)
        {  free(env);
           return -1;
        }
        argv[1]=strcpy(q,p);

        com.length=1;                   /* !=0, damit alles kopiert wird */
        for (r1=0;r1<128;com.command_tail[r1++]=' ');
        *(long *)(com.command_tail+IMAGIC)=ARGMAGIC;
        *(adr=com.command_tail+IARGV)=0xff; /* argc argv ans Ende der Zeile */
        *(adr+1)=(r1=((long)argv&0x00ff0000)>>16)?r1:(*adr&=~16,0xff);
        *(adr+2)=(r1=((long)argv&0x0000ff00)>>8)?r1:(*adr&=~8,0xff);
        *(adr+3)=(r1=(long)argv&0x000000ff)?r1:(*adr&=~4,0xff);
        *(adr+4)=(r1=(ARGC&0xff00)>>8)?r1:(*adr&=~2,0xff);
        *(adr+5)=(r1=ARGC&0x00ff)?r1:(*adr&=~1,0xff);
        com.command_tail[IARGC+2]='\0';     /* mit 0 abschlieen */
        erg=(int)Pexec(0,shell,&com,env);
        free(q);
        free(env);
        return erg;
}
#else

int shell(command)
char *command;
{
	return system(command);
}


#endif

