#define	FALSE       0
#define	TRUE        (!FALSE)

#define	LIBFILE		"layout.lib"
#define	KEYFILE		"keylay.lib"
#define	PINFILE		"pinlay.lib"
#define	LIBMAGIC	0xff65
#define	OBJMAGIC	0x601a
#define	NAMELEN		12

#define	MAGIC		0x0628
#define	TEMAGIC		0x0121

#define	XSCREEN		639		/* Bildschirmgroesse */
#define	YSCREEN		399
#define	XMAX		1260	/* Platinengroesse in Zoll/100 */
#define	YMAX		1260
#define	MAXDOTS		2000	/* default Anzahl der Loetpunkte */
#define	MAXLINES	5000	/* default Anzahl der Leiterbahnen */
#define	MAXLAYER	3		/* Anzahl der Layers */
#define	MAXSTACK	200		/* Groesse des track-stacks */
#define	MAXAREA		100		/* Anzahl der gefuellten Flaechen */
#define	MAXTEXT		200		/* Anzahl der Strings pro Layer */
#define	MAXELEM		500		/* Anzahl der Elemente */
#define	MAXPINS		300		/* Anzahl der Pins pro Element */
#define	STRLEN		16		/* Laenge der Strings */
#define	PNAMLEN		8		/* Laenge einer Pinbezeichnung */
#define	ENAMLEN		8		/* Laenge einer Elementbezeichnung */
#define	BLANKCHAR	8		/* Groesse der Luecke fuer Zeichen */
#define	PRTZOOM		3.001	/* maximaler Zoom fuer Drucker */
#define	PRTLDZ		1.501	/* Maximaler zoom fuer dots und tracks */
#define	ZOOMMAX		4		/* Vergroesserung */
#define	GRIDMAX		5		/* Raster */
#define	DOTTYPES	5		/* Anzahl der Typen */
#define	LINETYPES	10
#define	SEASIDE		2		/* Anzahl der Layer fuer Verbindungssuche */
#define	DOTHOLE		1.5		/* Lochdurchmesser */
#define	PRTHOLE		1.7
#define	TAN_CONST	0.207106781	/* 0.5 * tan(22.5) */
#define	ANGCONST1	0.577350269	/* tan(30) */
#define	ANGCONST2	1.732050808	/* tan(60) */

#define	ZOOMDEF		0		/* default zoom */
#define	DEFGRID		3		/* default Raster */
#define	STARTDOT	0		/* default Pin */
#define	STARTLINE	0

typedef struct
{
	int		x,			/* Nullpunkt */
			y,
			x1,			/* Eckkorrdinaten */
			y1,
			x2,
			y2,
			blk;		/* Flag fuer Block */
} Mv_Head;

typedef struct
{
	int		x1,			/* Groesse */
			y1,
			x2,
			y2,
			tx,			/* Textposition */
			ty,
			tr,			/* Textrotation */
			mx,			/* Markenposition */
			my;
	char	layname[ENAMLEN],
			listname[ENAMLEN];
} EleTyp;

typedef struct
{
	int		n,
			on;
	EleTyp	*e;
} Element;

typedef struct
{
	int		x,			/* Zeiger auf Koordinaten */
			y;
	int		s:6;		/* Zeiger auf Groesse */
	int		used:12;	/* Flag fuer "schon durchsucht" */
	char	name[PNAMLEN];
} DotTyp;

typedef struct
{
	int		n;
	int		on;
	DotTyp	*d;
} Dots;

typedef struct
{
	int		x1,			/* Zeiger auf Koordinaten */
			y1,
			x2,
			y2;
	int		d:6;		/* Richtung */
	int		s:6;		/* Groesse */
	int		used:4;		/* Flag fuer "schon dursucht" */
} LineTyp;

typedef struct
{
	int		x1,			/* Ecken */
			y1,
			x2,
			y2;
	int		used;		/* Flag fuer "schon durchsucht" */
} AreaTyp;

typedef struct			/* text */
{
	char	rot;		/* rotationsfaktor */
	char	mir;		/* gespiegelt */
	int		x,			/* Koordinaten */
			y,
			x1,			/* Block */
			y1,
			x2,
			y2;
	char	t[STRLEN];	/* string */
} TextTyp;
	
typedef struct
{
	int		n;
	int		na;
	int		nt;
	int		on;
	LineTyp	*l;
	AreaTyp	*a;
	TextTyp	*t;
} Lines;

typedef struct
{
	int		on,				/* Ein, Aus */
			x,
			y,
			nx,
			ny,
			sx,
			sy;
} Cursor;

typedef struct
{
	int		x,
			y;
} XY;

typedef struct
{
	double	x,
			y;
} FXY;

typedef struct
{
	long	x,
			y;
} LXY;

typedef struct				/* dot layout */
{
	XY		out[8],
			in[8];
} DLayout;

typedef struct				/* track layout */
{
	XY		out[8];
} LLayout;

typedef struct				/* dot layout */
{
	LXY		out[8],
			in[8];
} DPrintout;

typedef struct				/* track layout */
{
	LXY		out[8];
} LPrintout;

typedef struct
{
	int		x1,				/* Koordinaten */
			y1,
			x2,
			y2,
			on,				/* Ein, Aus */
			d;				/* Richtung */
} ActLine;

typedef struct
{
	int		x1,				/* Koordinaten */
			y1,
			x2,
			y2,
			on;				/* Ein, Aus */
} ActBlk;

typedef struct
{
	int		maxx,			/* Bildschirmgroesse */
			maxy,
			zoom,			/* Zoomfaktor */
			plane,			/* welche Seite */
			cmplane,		/* fuer copy und merge auf andere Layer */
			offx,			/* Offset */
			offy,
			grid,			/* Raster */
			cur,
			sx,				/* stack offset */
			sy,
			gfac,			/* Rasterfaktor */
			rot,			/* Rotation */
			mir,			/* Spiegelung fuer Texte */
			changed,		/* Flag geaenderte Daten */
			sorted,			/* Flag Daten sortiert */
			ctrl,			/* Flag Kontrollmode */
			stline,			/* Flag Statuszeile */
			dl,				/* dot layout */
			ll;				/* track layout */
} Info;

typedef struct
{
	int		minx,
			maxx,
			miny,
			maxy,
			offx,
			offy;
} Header;

typedef struct
{
	int		n;
	Header	h;
	LineTyp	*l;
} Keys;

typedef struct
{
	char	*name;		/* Name */
	int		deldot;		/* Flag zum unbenutzte Dots loeschen */
	long	offx,		/* Offset */
			offy,
			xres,		/* Ausloesung in dpi */
			yres,
			xmax,		/* Groesse */
			ymax;
	double	xy,			/* xres / yres */
			zoom,		/* Zoomfaktoren */
			lzoom,		/* linezoom */
			dzoom,		/* dot zoom */
			zoomx,
			zoomy;
	void	*buffer;	/* Datenpuffer */
	long	buf_len;	/* Pufferlaenge */
} PData;

typedef struct
{
	char			name[NAMELEN];
	int				unknown_1;
	unsigned		date;
	unsigned		time;
	long			unknown_2;
	unsigned long	length;
	int				unknown_3;
} LibHead;

typedef struct
{
	int				magic;
	unsigned long	text_len,
					data_len,
					bss_len,
					symbol_len;
	char			unknown[10];
} ObjHead;

#define	MIN(a,b)	(((a) < (b)) ? (a) : (b))
#define	MAX(a,b)	(((a) > (b)) ? (a) : (b))

#define	D_IN_BLK(p)	((p)->x >= act_block.x1 && (p)->x <= act_block.x2 && \
					 (p)->y >= act_block.y1 && (p)->y <= act_block.y2)
#define	LAT_IN_BLK(l)	((l)->x1 >= act_block.x1 && (l)->x1 <= act_block.x2 && \
					 	(l)->x2 >= act_block.x1 && (l)->x2 <= act_block.x2 && \
					 	(l)->y1 >= act_block.y1 && (l)->y1 <= act_block.y2 && \
					 	(l)->y2 >= act_block.y1 && (l)->y2 <= act_block.y2)
#define	D_MM_HEAD(p)	head.minx = MIN(head.minx, (p)->x); \
						head.miny = MIN(head.miny, (p)->y); \
						head.maxx = MAX(head.maxx, (p)->x); \
						head.maxy = MAX(head.maxy, (p)->y);
#define	LAT_MM_HEAD(l)  head.minx = MIN(head.minx, (l)->x1); \
						head.miny = MIN(head.miny, (l)->y1); \
						head.maxx = MAX(head.maxx, (l)->x1); \
						head.maxy = MAX(head.maxy, (l)->y1); \
						head.minx = MIN(head.minx, (l)->x2); \
						head.miny = MIN(head.miny, (l)->y2); \
						head.maxx = MAX(head.maxx, (l)->x2); \
						head.maxy = MAX(head.maxy, (l)->y2);

