/* license: GNU GPL 2+ (see ../COPYING) */
#include <linea.h>
#include "l_macro.h"

#warning TODO

static unsigned char shadow[1024*YSCREEN]; /* pretend we have 1024 pixels in width for faster mul */

static int getpix(int x, int y)
{
	unsigned long offs;

	if ((x < 0) || (x > XSCREEN))
		return -1;

	if ((y < 0) || (y > YSCREEN))
		return -1;

	offs = y;
	offs <<= 10; /* *1024 */
	offs += x;
	return shadow[offs];
}

static void setpix(int x, int y, int val)
{
	unsigned long offs;

	if ((x < 0) || (x > XSCREEN))
		return;

	if ((y < 0) || (y > YSCREEN))
		return;

	offs = y;
	offs <<= 10; /* *1024 */
	offs += x;
	shadow[offs] = val;

	if (!val)
		SDL_SetRenderDrawColor(Vdiesc->rend, 255, 255, 255, 255);
	else
		SDL_SetRenderDrawColor(Vdiesc->rend, 0, 0, 0, 255);
	SDL_RenderDrawPoint(Vdiesc->rend, x, y);
}

static void togglepix(int x, int y)
{
	unsigned long offs;

	if ((x < 0) || (x > XSCREEN))
		return;

	if ((y < 0) || (y > YSCREEN))
		return;

	offs = y;
	offs <<= 10; /* *1024 */
	offs += x;
	shadow[offs] = !shadow[offs];

	if (!shadow[offs])
		SDL_SetRenderDrawColor(Vdiesc->rend, 255, 255, 255, 255);
	else
		SDL_SetRenderDrawColor(Vdiesc->rend, 0, 0, 0, 255);
	SDL_RenderDrawPoint(Vdiesc->rend, x, y);
}

static void putrect(int x1, int y1, int x2, int y2, int val)
{
	int x, y, w;
	unsigned long offs;
	SDL_Rect r;

	if (x2 < x1) {
		x = x1;
		x1 = x2;
		x2 =x;
	}

	if (y2 < y1) {
		y = y1;
		y1 = y2;
		y2 = y;
	}

	if (x1 < 0)
		x1 = 0;
	if (y1 < 0)
		y1 = 0;
	if (x1 > XSCREEN)
		x1 = XSCREEN;
	if (y1 > YSCREEN)
		y1 = YSCREEN;

	if ((x2 < 0) || (y2 < 0))
		return;
	if ((x1 > XSCREEN) || (y1 > YSCREEN))
		return;

	if (x2 > XSCREEN)
		x2 = XSCREEN;
	if (y2 > YSCREEN)
		y2 = YSCREEN;

	offs = y1;
	offs <<= 10; /* *1024 */
	offs += x1;
	w = x2 - x1 + 1;
	for(y = y1; y <= y2; y++) {
		memset(shadow+offs, val, w);
		offs += 1024;
	}

	if (!val)
		SDL_SetRenderDrawColor(Vdiesc->rend, 255, 255, 255, 255);
	else
		SDL_SetRenderDrawColor(Vdiesc->rend, 0, 0, 0, 255);

	r.x = x1;
	r.y = y1;
	r.w = w;
	r.h = y2 - y1 + 1;
	SDL_RenderFillRect(Vdiesc->rend, &r);
}

void o_scrinit(void *scr_p, void *font_p)
{
}

void o_clrscr(void)
{
	SDL_SetRenderDrawColor(Vdiesc->rend, 255, 255, 255, 255);
	SDL_RenderFillRect(Vdiesc->rend, NULL);
	memset(shadow, 0, sizeof(shadow));
}

void o_clrstat(void)
{
	static const SDL_Rect statr = {0, YSCREEN-7, XSCREEN, 8 };
	SDL_SetRenderDrawColor(Vdiesc->rend, 255, 255, 255, 255);
	SDL_RenderFillRect(Vdiesc->rend, &statr);
}

void o_cursor(int xc, int yc)
{
	SDL_Rect r;
	int x, y;

	for(x = 1; x < XSCREEN; x+=2) {
		if (x == xc)
			continue;
		togglepix(x, yc);
	}

	for(y = 1; y < YSCREEN; y+=2)
		togglepix(xc, y);
}

void o_status(char *str)
{
	int x;
	for(x = 0; *str != '\0'; x += font_draw_char(x, YSCREEN-7, *str), str++) ;
}

void o_hordot(int x1, int x2, int y)
{
	int x;

	if (x1 > x2) {
		x = x1;
		x1 = x2;
		x2 = x;
	}

	if (x1 % 2)
		x1++;

	for(x = x1; x < x2; x+=2)
		togglepix(x, y);
}

void o_verdot(int x, int y1, int y2)
{
	int y;

	if (y1 > y2) {
		y = y1;
		y1 = y2;
		y2 = y;
	}

	if (y1 % 2)
		y1++;

	for(y = y1; y < y2; y+=2)
		togglepix(x, y);
}

void o_dot45(int x1, int y1, int x2, int y2)
{
	int x, y, dy;

	if (x1 > x2) {
		x = x1;
		x1 = x2;
		x2 = x;
		y = y1;
		y1 = y2;
		y2 = y;
	}

	dy = (y2 > y1) ? +2 : -2;

	for(x = x1, y = y1; x < x2; x+=2,y+=dy)
		togglepix(x, y);
}

void o_urdot(int x1, int y1, int x2, int y2)
{
	o_dot45(x1, y1, x2, y2);
}

void o_drdot(int x1, int y1, int x2, int y2)
{
	o_dot45(x1, y1, x2, y2);
}

static void draw_line(int x1, int x2, int y, int val)
{
	int x;

	if (x1 > x2) {
		x = x1;
		x1 = x2;
		x2 = x;
	}

	for(x = x1; x <= x2; x++)
		setpix(x, y, val);
}

void o_chline(int x1, int x2, int y)
{
	draw_line(x1, x2, y, 1);
}

void o_chdlin(int x1, int x2, int y)
{
	draw_line(x1, x2, y, 0);
}



/* draw solid rect */
void o_cblk(int x1, int y1, int x2, int y2)
{
	putrect(x1, y1, x2, y2, 1);
}

/* del solid rect */
void o_cdblk(int x1, int y1, int x2, int y2)
{
	putrect(x1, y1, x2, y2, 0);
}

