/*****************************************************

		Platinen Layout fr ATARI ST in TURBO C
					Thomas Nau (c) 1990
	license: GNU GPL 2+ (see ../COPYING)

*****************************************************/

/* Standart INCLUDE files */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <tos.h>

#define L_SEARCH

#include "l_incl.h"     /* alle anderen INCLUDE Files */

/****************************************************

		Vergleichsfunktion fuer x1

****************************************************/
int	cmp_x1(o1, o2)
const void	*o1, *o2;
{
	const int *p1 = o1, *p2 = o2;
	return((l_track[sortside].l + (*p2))->x1 -
		   (l_track[sortside].l + (*p1))->x1);
}

/****************************************************

		Vergleichsfunktion fuer x2

****************************************************/
int	cmp_x2(o1, o2)
const void	*o1, *o2;
{
	const int *p1 = o1, *p2 = o2;
	return((l_track[sortside].l + (*p1))->x2 -
		   (l_track[sortside].l + (*p2))->x2);
}

/****************************************************

		liefert ersten Index <= x

****************************************************/
int bs_x1(x, side)
register	int		x, side;
{
	register	int		l, r, m;
	register	LineTyp	*lp;

	lp = l_track[side].l;
	l = 0;
	r = l_track[side].n -1;
	m = (r+l)/2;
	while (l < r)
	{
		if ((lp +x1_sort[side][m])->x1 > x)
			l = m+1;
		else
			r = m;
		m = (l+r)/2;
	}
	return(m);
}

/****************************************************

		liefert ersten Index >= x

****************************************************/
int bs_x2(x, side)
register	int		x, side;
{
	register	int		l, r, m;
	register	LineTyp	*lp;

	lp = l_track[side].l;
	l = 0;
	r = l_track[side].n -1;
	m = (r+l)/2;
	while (l < r)
	{
		if ((lp +x2_sort[side][m])->x2 < x)
			l = m+1;
		else
			r = m;
		m = (l+r)/2;
	}
	return(m);
}

/****************************************************

		Arrays sortieren

****************************************************/
void sort_array()
{
	o_clrstat();
	o_status("running sort routine...");
	for (sortside = 0; sortside < SEASIDE; sortside++)
	{
		qsort(&x1_sort[sortside][0], l_track[sortside].n, sizeof(int), cmp_x1);
		qsort(&x2_sort[sortside][0], l_track[sortside].n, sizeof(int), cmp_x2);
	}
	o_clrstat();
	info.sorted = TRUE;
}

/****************************************************

		used Flag loeschen

****************************************************/
void del_used()
{
	register	int		i, side;

	for (i = 0; i < d_data.n; i++)
		(d_data.d +i)->used = FALSE;
	for (side = 0; side < SEASIDE; side++)
	{
		for (i = 0; i < l_track[side].n; i++)
			(l_track[side].l +i)->used = FALSE;
		for (i = 0; i < l_track[side].na; i++)
			(l_track[side].a +i)->used = FALSE;
	}
}

/****************************************************

		sucht alle Verbindungen zu Punkt i

****************************************************/
int is_connected(dp, side)
DotTyp *dp;
int	 side;
{
	register	LineTyp	*lp;
	register	AreaTyp	*ap;
	register	int		i, d, dl, found, y;

	d = sd_width[dp->s];
	lp = l_track[side].l;
	for (i = 0; i < l_track[side].n; i++, lp++)
	{
		dl = d +sl_width[lp->s];
		if (lp->x1 -dl <= dp->x &&
			lp->x2 +dl >= dp->x)
		{
			switch(lp->d)
			{
				case 0 : found = ((dp->y >= lp->y1 -dl &&
								 dp->y <= lp->y2 +dl) ? TRUE : FALSE);
						 break;
				case 1 : y = dp->x -lp->x1 + lp->y1;
						 found = ((dp->y >= lp->y1 -dl &&
						 		 dp->y <= lp->y2 +dl &&
						 		 abs(dp->y -y) <= dl) ? TRUE : FALSE);
						 break;
				case 2 : found = ((dp->y >= lp->y1 -dl &&
								 dp->y <= lp->y2 +dl) ? TRUE : FALSE);
						 break;
				case 3 : y = -dp->x +lp->x1 + lp->y1;
						 found = ((dp->y >= lp->y2 -dl &&
						 		 dp->y <= lp->y1 +d &&
						 		 abs(dp->y -y) <= dl) ? TRUE : FALSE);
						 break;
				default: found = FALSE;;
						 break;
			}
			if (found)
				return(TRUE);
		}
	}
	ap = l_track[side].a;
	for (i = 0; i < l_track[side].na; i++, ap++)
		if (dp->x >= ap->x1 -d &&
			dp->x <= ap->x2 +d &&
			dp->y >= ap->y1 -d &&
			dp->y <= ap->y2 +d)
			return(TRUE);
	return(FALSE);
}

/****************************************************

		initialisiert suchen

****************************************************/
int rtr_connect(n, stat)
int		n;
char	*stat;
{
	register	int	side, i;

	for (side = 0; side < SEASIDE; side++)
	{
		act_track[side] = act_area[side] = 0;
		nr_track[side] = nr_area[side] = 0;
	}
	if (!info.sorted)
	{
		for (side = 0; side < SEASIDE; side ++)
			for (i = 0; i < l_track[side].n; i++)
				x1_sort[side][i] = x2_sort[side][i] = i;
		sort_array();
	}
	nr_dot = 1;
	act_dot = 0;
	(d_data.d +n)->used = TRUE;
	dot_list[act_dot] = n;
	o_clrstat();
	o_status(stat);
	while (act_dot < nr_dot)
	{
		con_dot();
		act_dot++;
	}
	o_clrstat();
	return(nr_dot);
}

/****************************************************

		sucht alle Verbindungen zu Punkt i

****************************************************/
void con_first(dp)
DotTyp *dp;
{
	register	LineTyp	*lp;
	register	AreaTyp	*ap;
	register	int		i, d, dl, found, side, y;

	d = sd_width[dp->s];
	for (side = 0; side < SEASIDE; side++)
	{
		lp = l_track[side].l;
		for (i = 0; i < l_track[side].n; i++, lp++)
			if (!lp->used)
			{
				dl = d +sl_width[lp->s];
				if (lp->x1 -dl <= dp->x &&
					lp->x2 +dl >= dp->x)
				{
					switch(lp->d)
					{
						case 0 : found = ((dp->y >= lp->y1 -dl &&
										 dp->y <= lp->y2 +dl) ? TRUE : FALSE);
								 break;
						case 1 : y = dp->x -lp->x1 + lp->y1;
								 found = ((dp->y >= lp->y1 -dl &&
								 		 dp->y <= lp->y2 +dl &&
								 		 abs(dp->y -y) <= dl) ? TRUE : FALSE);
								 break;
						case 2 : found = ((dp->y >= lp->y1 -dl &&
										 dp->y <= lp->y2 +dl) ? TRUE : FALSE);
								 break;
						case 3 : y = -dp->x +lp->x1 + lp->y1;
								 found = ((dp->y >= lp->y2 -dl &&
								 		 dp->y <= lp->y1 +d &&
								 		 abs(dp->y -y) <= dl) ? TRUE : FALSE);
								 break;
						default: found = FALSE;;
								 break;
					}
					if (found)
					{
						lp->used = TRUE;
						track_list[side][nr_track[side]++] = i;
					}
				}
			}
		ap = l_track[side].a;
		for (i = 0; i < l_track[side].na; i++, ap++)
			if (!ap->used &&
				dp->x >= ap->x1 -d &&
				dp->x <= ap->x2 +d &&
				dp->y >= ap->y1 -d &&
				dp->y <= ap->y2 +d)
			{
				ap->used = TRUE;
				area_list[side][nr_area[side]++] = i;
			}
	}
}

/****************************************************

		sucht alle Verbindungen von Punkten zu Linien und Flaechen

****************************************************/
void con_dot()
{
	register	DotTyp		*dp;
				int			side,
							done;
	static		int			last_act[SEASIDE];

	for (side = 0; side < SEASIDE; side++)
		last_act[side] = act_track[side];
	dp = d_data.d +dot_list[act_dot];
	con_first(dp);
	do
	{
		for (side = 0; side < SEASIDE; side++)
		{
			while (act_track[side] < nr_track[side])
			{
				con_line(l_track[side].l +track_list[side][act_track[side]], side);
				act_track[side]++;
			}
			while (act_area[side] < nr_area[side])
			{
				con_area(l_track[side].a +area_list[side][act_area[side]], side);
				act_area[side]++;
			}
		}
		done = TRUE;
		for (side = 0; side < SEASIDE; side++)
		{
			done = done && 
				   act_track[side] >= nr_track[side] &&
				   act_area[side] >= nr_area[side];
		}
	} while (!done);
	con_all_dots(last_act);
}

/****************************************************

		sucht alle Verbindungen von Linien zu Linien und Flaechen

****************************************************/
void con_line(lp, side)
register	LineTyp	*lp;
			int		side;
{
	register	LineTyp		*lp1, *lpn, *lps;
	register	int			*sp;
	register	AreaTyp		*ap;
	register	int			i, y, x1d, y1d, x2d, y2d;
				int			pos1, pos2, maxdiff, d, found, dl, x;

	ap = l_track[side].a;
	dl = sl_width[lp->s];
	maxdiff = dl +sl_width[LINETYPES-1];
	pos1 = bs_x2(lp->x1 -maxdiff, side);
	pos2 = bs_x1(lp->x2 +maxdiff, side);
	if (pos1 > pos2)
	{
		i = pos1;
		sp = &x2_sort[side][0];
	}
	else
	{
		i = pos2;
		sp = &x1_sort[side][0];
	}
	sp += i;
	while (i < l_track[side].n)
	{
		lp1 = l_track[side].l + (*sp);
		if (!lp1->used)
		{
			d = dl +sl_width[lp1->s];
			if (lp->d >= lp1->d)
			{
				lpn = lp1;
				lps = lp;
			}
			else
			{
				lpn = lp;
				lps = lp1;
			}
			if (lps->x1 <= (x2d = lpn->x2 +d) &&
				lps->x2 >= (x1d = lpn->x1 -d))
			{
				y2d = lpn->y2 +d;
				y1d = lpn->y1 -d;
				switch(lpn->d)
				{
					case 0 : switch(lps->d)
							 {
							 	case 0 : found = ((lps->y1 <= y2d &&
							 					 lps->y2 >= y1d) ? TRUE : FALSE);
										 break;
							 	case 1 : y = lpn->x1 -lps->x1 +lps->y1;
					 					 found = ((y1d <= y &&
							 			 		 y <= y2d) ? TRUE : FALSE);
										 break;
							 	case 2 : found = ((y1d <= lps->y1 &&
							 					 lps->y1 <= y2d) ? TRUE : FALSE);
										 break;
							 	case 3 : y = -lpn->x1 +lps->x1 +lps->y1;
							 			 found = ((y1d <= y &&
					 			 				 y <= y2d) ? TRUE : FALSE);
										 break;
							 	default: found = (FALSE);
										 break;
							 }
							 break;
					case 1 : switch(lps->d)
							 {
							 	case 1 : y = lpn->x1 -lps->x1 +lps->y1;
				 						 found = ((y1d <= y &&
							 			 		 y <= lpn->y1 +d) ? TRUE : FALSE);
										 break;
							 	case 2 : x = lps->y1 -lpn->y1 +lpn->x1;
							 			 found = ((y1d <= lps->y1 &&
							 			 		 lps->y1 <= y2d &&
				 						 		 lps->x1 -d <= x &&
				 			 					 x <= lps->x2 +d) ? TRUE : FALSE);
										 break;
							 	case 3 : x = (lpn->x1 +lps->x1 +lps->y1 -lpn->y1) /2;
							 			 y = x -lpn->x1 +lpn->y1;
							 			 found = ((lpn->x1 -lpn->d <= x &&
						 				 		 x <= lpn->x2 +lpn->d &&
					 					 		 lpn->y2 -lpn->d <= y &&
							 			 		 y <= lpn->y1 +lpn->d &&
						 				 		 lps->x1 -lps->d <= x &&
						 				 		 x <= lps->x2 +lps->d &&
					 					 		 lps->y2 -lps->d <= y &&
							 			 		 y <= lps->y1 +lps->d) ? TRUE : FALSE);
										 break;
							 	default: found = (FALSE);
										 break;
							 }
							 break;
					case 2 : switch(lps->d)
							 {
							 	case 2 : found = ((abs(lpn->y1 -lps->y1) <= d) ? TRUE : FALSE);
										 break;
							 	case 3 : x = lps->x1 +lps->y1 -lpn->y1;
				 						 found = ((lps->y2 -d <= lpn->y1 &&
						 				 		 lpn->y1 <= lps->y1 +d &&
							 			 		 x1d <= x &&
							 			 		 x <= x2d) ? TRUE : FALSE);
										 break;
							 	default: found = (FALSE);
										 break;
							 }
							 break;
					case 3 : switch(lps->d)
							 {
							 	case 3 : y = -lpn->x1 +lps->x1 +lps->y1;
							 			 found = ((y1d <= y &&
				 						 		 y <= lpn->y1 +d) ? TRUE : FALSE);
										 break;
							 	default: found = (FALSE);
										 break;
							 }
							 break;
					default: found = (FALSE);
							 break;
				}
				if (found)
				{
					lp1->used = TRUE;
					track_list[side][nr_track[side]++] = *sp;
				}
			}
		}
		i++;
		sp++;
	}
	for (i = 0; i < l_track[side].na; i++, ap++)
	{
		if (!ap->used)
			if (line_area(lp, ap))
			{
				ap->used = TRUE;
				area_list[side][nr_area[side]++] = i;
			}
	}
}

/****************************************************

		sucht alle Verbindungen von Flaechen zu Linien und Flaechen

****************************************************/
void con_area(ap, side)
int		side;
AreaTyp	*ap;
{
	register	LineTyp		*lp;
	register	AreaTyp		*ap1;
	register	int			i;

	ap1 = l_track[side].a;
	lp = l_track[side].l;
	for (i = 0; i < l_track[side].n; i++, lp++)
	{
		if (!lp->used)
			if (line_area(lp, ap))
			{
				lp->used = TRUE;
				track_list[side][nr_track[side]++] = i;
			}
	}
	for (i = 0; i < l_track[side].na; i++, ap1++)
	{
		if (!ap1->used)
			if (area_area(ap, ap1))
			{
				ap1->used = TRUE;
				area_list[side][nr_area[side]++] = i;
			}
	}
}

/****************************************************

		Schnitt Linie Flaeche ?

****************************************************/
int line_area(lp, ap)
LineTyp	*lp;
AreaTyp	*ap;
{
	register	int		x1, y1, x2, y2;
				int		d;

	x1 = lp->x1;
	y1 = lp->y1;
	x2 = lp->x2;
	y2 = lp->y2;
	d = sl_width[lp->d]/2;
	switch(lp->d)
	{
		case 0 : return((x1 >= ap->x1 -d &&
						 x1 <= ap->x2 +d &&
						 y1 <= ap->y2 +d &&
						 y2 >= ap->y1 -d) ? TRUE : FALSE);
		case 1 : if (x1 < ap->x1 -d)
				 {
				 	y1 += (ap->x1 -d -x1);
				 	x1 = ap->x1 -d;
				 }
				 if (x2 > ap->x2 +d)
				 {
				 	y2 -= (x2 -ap->x2 -d);
				 	x2 = ap->x2 +d;
				 }
				 if (y1 < ap->y1 -d)
				 {
				 	x1 += (ap->y1 -d -y1);
				 	y1 = ap->y1 -d;
				 }
				 if (y2 > ap->y2 +d)
				 {
				 	x2 -= (y2 -ap->y2 -d);
				 	y2 = ap->y2 +d;
				 }
				 return((x1 <= x2 && y1 <= y2) ? TRUE : FALSE);
		case 2 : return((y1 >= ap->y1 -d &&
						 y1 <= ap->y2 +d &&
						 x1 <= ap->x2 +d &&
						 x2 >= ap->x1 -d) ? TRUE : FALSE);
		case 3 : if (x1 < ap->x1 -d)
				 {
				 	y1 -= (ap->x1 -d -x1);
				 	x1 = ap->x1 -d;
				 }
				 if (x2 > ap->x2 +d)
				 {
				 	y2 += (x2 -ap->x2 -d);
				 	x2 = ap->x2 +d;
				 }
				 if (y2 < ap->y1 -d)
				 {
				 	x2 -= (ap->y1 -d -y2);
				 	y2 = ap->y1 -d;
				 }
				 if (y1 > ap->y2 +d)
				 {
				 	x1 += (y1 -ap->y2 -d);
				 	y1 = ap->y2 +d;
				 }
				 return((x1 <= x2 && y2 <= y1) ? TRUE : FALSE);
		default: return(FALSE);
	}
}

/****************************************************

		Schnitt Flaeche Flaeche ?

****************************************************/
int area_area(ap, ap1)
AreaTyp	*ap, *ap1;
{
	return((ap1->x2 >= ap->x1 &&
			ap1->x1 <= ap->x2 &&
			ap1->y2 >= ap->y1 &&
			ap1->y1 <= ap->y2) ? TRUE : FALSE);
}

/****************************************************

		setzt alle Punkte die auf gefundenen Linien oder Flaechen liegen

****************************************************/
void con_all_dots(last_act)
int	*last_act;
{
	register	int		i, j, d, side, y, dl, found;
	register	DotTyp	*dp;
	register	AreaTyp	*ap;
	register	LineTyp	*lp;

	dp = d_data.d;
	for (j = 0; j < d_data.n; j++, dp++)
	{
		dl = sd_width[dp->s];
		for (side = 0; side < SEASIDE && !dp->used; side++)
		{
			for (i = *(last_act+side); i < nr_track[side]; i++)
			{
				lp = l_track[side].l +track_list[side][i];
				d = dl +sl_width[lp->s];
				if (dp->x >= lp->x1 -d &&
					dp->x <= lp->x2 +d)
				{
					switch(lp->d)
					{
						case 0 : found = ((dp->y >= lp->y1 -dl &&
										 dp->y <= lp->y2 +dl) ? TRUE : FALSE);
								 break;
						case 1 : y = dp->x -lp->x1 + lp->y1;
								 found = ((dp->y >= lp->y1 -dl &&
								 		 dp->y <= lp->y2 +dl &&
								 		 abs(dp->y -y) <= dl) ? TRUE : FALSE);
								 break;
						case 2 : found = ((dp->y >= lp->y1 -dl &&
										 dp->y <= lp->y2 +dl) ? TRUE : FALSE);
								 break;
						case 3 : y = -dp->x +lp->x1 + lp->y1;
								 found = ((dp->y >= lp->y2 -dl &&
								 		 dp->y <= lp->y1 +d &&
								 		 abs(dp->y -y) <= dl) ? TRUE : FALSE);
								 break;
						default: found = FALSE;;
								 break;
					}
					if (found)
					{
						dp->used = TRUE;
						dot_list[nr_dot++] = j;
						i = nr_track[side];
					}
				}
			}
			for (i = 0; i < nr_area[side]; i++)
			{
				ap = l_track[side].a +area_list[side][i];
				if (dp->x >= ap->x1 -dl &&
					dp->x <= ap->x2 +dl &&
					dp->y >= ap->y1 -dl &&
					dp->y <= ap->y2 +dl)
				{
					dp->used = TRUE;
					dot_list[nr_dot++] = j;
					i = nr_area[side];
				}
			}
		}
	}
}

/****************************************************

		gibt alle Verbindungen zu allen Elementen aus

****************************************************/
void list_all_con(fname)
char	*fname;
{
	int		i, j;
	FILE	*fp;
	EleTyp	*et;

	if (Fattrib(fname, 0, 0) >= 0)
		if (!get_ok("file exists, override ? "))
			return;
	if ((fp = fopen(fname, "w")) == NULL)
	{
		say_error("write-open error");
		return;
	}
	et = element.e;
	del_used();
	for (i = 0; i < element.n; i++, et++)
	{
		list_elem(et, fp);
		for (j = 0; j < 6; j++)
			fprintf(fp, "----------");
		fprintf(fp, "\n\n");
	}
	fclose(fp);
}

/****************************************************

		gibt alle Verbindungen zu Element aus

****************************************************/
void list_connect(fname)
char	*fname;
{
	EleTyp	*et;
	int		i, x, y;
	FILE	*fp;

	act_coord(&x, &y);
	et = element.e;
	for (i = 0; i < element.n; i++, et++)
	{
		if (x >= et->x1 && x <= et->x2 &&
			y >= et->y1 && y <= et->y2) break;
	}
	if (i >= element.n)
	{
		say_error("no matching elements...");
		return;
	}
	if (Fattrib(fname, 0, 0) >= 0)
		if (!get_ok("file exists, override ? "))
			return;
	if ((fp = fopen(fname, "w")) == NULL)
	{
		say_error("write-open error");
		return;
	}
	del_used();
	list_elem(et, fp);
	fclose(fp);
}

/****************************************************

		gibt alle Verbindungen zu Element aus

****************************************************/
void list_elem(et, fp)
EleTyp	*et;
FILE	*fp;
{
	register	int		i, j, k;
	register	DotTyp	*dt, *dt1;
	register	EleTyp	*et1, *et2;
				char	s[81];
				char	np[ENAMLEN+1];

	dt = d_data.d;
	for (i = 0; i < d_data.n; i++, dt++)
	{
		sprintf(s, "processing pin %d of %d", i+1, d_data.n);
		if (dt->x >= et->x1 && dt->x <= et->x2 &&
			dt->y >= et->y1 && dt->y <= et->y2)
		{
			if (!info.ctrl)
				del_used();
			if (dt->used)
			{
				if (!dt->name[0])
					fprintf(fp, " (%i,%i)(%s) checked earlier\n", 
							dt->x, dt->y, et->listname);
				else
					fprintf(fp, " %s(%s) checked earlier\n",
							dt->name, et->listname);
				continue;
			}	
			rtr_connect(i, s);
			if (nr_dot > 1)
			{
				for (j = 0; j < nr_dot; j++)
				{
					dt1 = d_data.d +dot_list[j];
					et2 = NULL;
					for (k = 0, et1 = element.e; k < element.n; k++, et1++)
					{
						if (dt1->x >= et1->x1 && dt1->x <= et1->x2 &&
							dt1->y >= et1->y1 && dt1->y <= et1->y2)
						{
							if (et2 == NULL)
								et2 = et1;
							else
							{
								if ((et1->x2 -et1->x1 <= et2->x2 -et2->x1) &&
									(et1->y2 -et1->y1 <= et2->y2 -et2->y1))
									et2 = et1;
							}
						}
					}
					if (et2 == NULL)
						strcpy(np, "unknown");
					else
						strcpy(np, et2->listname);
					if (!dt1->name[0])
						fprintf(fp, " - (%i,%i)(%s)", 
								dt1->x, dt1->y, np);
					else
						fprintf(fp, " - %s(%s)", dt1->name, np);
				}
				fprintf(fp, "\n");
			}
			else
				if (!dt->name[0])
					fprintf(fp, " (%i,%i)(%s) no connection\n", 
							dt->x, dt->y, et->listname);
				else
					fprintf(fp, " %s(%s) no connection\n",
							dt->name, et->listname);
		}
	}
	fprintf(fp, "\n");
}
