/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/*                                                                     */
/*       Layout Druckprogramm fuer FX80, LQ850 und HP Laserjet         */
/*	 	 und Kompatible	       									       */
/*                (c)   B.u.H.Daeubler 25.11.1990                      */
/*	license: GNU GPL 2+ (see ../COPYING)
/*                                                                     */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

#include <stdio.h>
#include <stdarg.h>
#include <ctype.h>
#include <stdlib.h>
#include <string.h>
#include <main.h>
#include <fatal.h>
#ifndef UNIX
#	include <lp.h>
#endif


#define USAGE fprintf(stderr,"usage : %s [-si][-qlfj] file\n",fatal_name),\
		exit(1)

#define UNKNOWN -1
#define FX 0
#define LQ 1
#define JET 2

#define PRNBUF 16000

#ifndef UNIX
    typedef int SIXTEENBIT;
#else
    typedef short SIXTEENBIT;
#endif

static int printer=UNKNOWN;
static int qflag=0, sflag=0, iflag=0;


#ifndef UNIX
#	define write_buf(buf,nchs)		(flush_prn(),lpwrite(buf, nchs))
#	define init_prn()
#	define flush_prn()				(lpwrite(ch_buf,buf_pointer),buf_pointer=0)

	static char ch_buf[PRNBUF];
	static int buf_pointer=0;

static void put_prn(int ch)
{	if (buf_pointer==PRNBUF) flush_prn();
	ch_buf[buf_pointer++]=ch;
}
#else
#	define put_prn(ch)				fputc(ch,stdout)
#	define write_buf(buffer,nchs)	fwrite(buffer,sizeof(char),nchs,stdout)
#	define init_prn()				setvbuf(stdout,NULL,_IOFBF,PRNBUF)
#	define flush_prn()				fflush(stdout)
#endif

static void put_str(register char *p)
{       while (*p) put_prn(*p++);
}

static void put_fstr(char *fmt,...)
{		va_list argp;
		static char fmtbuf[80];
		va_start(argp,fmt);

		vsprintf(fmtbuf,fmt,argp);
		put_str(fmtbuf);
		va_end(argp);
}

static int lq_line(register char *p,register int skip,register int size_x)
{       register int i,j;

        for (i=size_x-1,j=3*i;i>=0 && !p[j] && !p[j+1] && !p[j+2];i--,j-=3);
        if (i>=0)
        {  while (skip>255) put_str("\033J\377"),skip-=255;
           if (skip) put_str("\033J");put_prn(skip);
           skip=0;
           i++;j=0;
           while (!p[0] && !p[1] && !p[2]) p+=3,j++,i--;
           put_str("\033$");put_prn(j/3);put_prn((j/3)>>8);
           i+=j%3;p-=(j%3)*3;
           put_str("\033*\047");put_prn(i);put_prn(i>>8);
           while (i--) put_prn(*p++),put_prn(*p++),put_prn(*p++);
           put_prn('\r');
        }
        return skip;
}

static int lp_line(register char *p,register int skip,register int size_x)
{       register int i,j;

        for (i=size_x-1;i>=0 && !p[i];i--);
        if (i>=0)
        {  while (skip>255) put_str("\033J\377"),skip-=255;
           if (skip) put_str("\033J");put_prn(skip);
           skip=0;
           i++;j=0;
           while (!*p) p++,j++,i--;
           put_str("\033l");put_prn(j/12);
           i+=j%12;p-=j%12;
           put_str("\033*\03");put_prn(i);put_prn(i>>8);
           while (i--) put_prn(*p++);
           put_prn('\r');
        }
        return skip;
}

static int lp_evenodd(register char *p,register int skip,int odd,register int size_x)
{       register int i,j,k;

        for (i=size_x-1,k=(i+odd)&1;i>=0 && (k || !p[i]);i--,k^=1);
        if (i>=0)
        {  while (skip>255) put_str("\033J\377"),skip-=255;
           if (skip) put_str("\033J");put_prn(skip);
           skip=0;
           i++;j=0,k=odd;
           while (k || !*p) p++,j++,i--,k^=1;
           put_str("\033l");put_prn(j/12);
           i+=j%12,p-=j%12,k=(j%12)&1;
           put_str("\033*\03");put_prn(i);put_prn(i>>8);
           while (i--) put_prn((k^=1)?*p++:(p++,0));
           put_prn('\r');
        }
        return skip;
}

static void lp_buf(register char *buffer,register int lines,register int size_x)
{       register int i;
        register int skip=0;

        lines=(lines+23)/24;
        if (printer==FX)
        {   if (qflag)
                while (lines--)
                {   skip=lp_evenodd(buffer,skip,0,size_x);
                    skip=lp_evenodd(buffer,skip,1,size_x)+1;buffer+=size_x;
                    skip=lp_evenodd(buffer,skip,0,size_x);
                    skip=lp_evenodd(buffer,skip,1,size_x)+1;buffer+=size_x;
                    skip=lp_evenodd(buffer,skip,0,size_x);
                    skip=lp_evenodd(buffer,skip,1,size_x)+22;buffer+=size_x;
                }
            else
                while (lines--)
                {   skip=lp_line(buffer,skip,size_x)+1;buffer+=size_x;
                    skip=lp_line(buffer,skip,size_x)+1;buffer+=size_x;
                    skip=lp_line(buffer,skip,size_x)+22;buffer+=size_x;
                }
            while (skip>255) put_str("\033J\377"),skip-=255;
            if (skip) put_str("\033J");put_prn(skip);
        }
        else
        {   while (lines--)
               skip=lq_line(buffer,skip,size_x)+24,buffer+=3*size_x;
            while (skip>255) put_str("\033J\377"),skip-=255;
            if (skip) put_str("\033J");put_prn(skip);
        }
}

void lp_page(void)
{       if (printer!=JET) put_str("\r\014\033@");
		else put_str("\033*rB\014");
        flush_prn();        
}

void init_page(void)
{    	switch(printer)
		{	case FX: put_str("\033@\033!\05");break;
			case LQ: put_str("\033@");
					 if (qflag) put_str("\033U1");
					 break;
			case JET:put_str("\033*t300R\033*r0A");break;
		}
}

static void clear_buffer(register char *buffer,register int size_x)
{       size_x*=3;
        while (size_x--) *buffer++='\0';
}

static char *get_ptr(int *inc,int *mask,char *buffer,register int size_x,
                    register int y)
{       register int d,m;

        if (printer==FX)
        {   d=y/3,m=y%3;
            *inc=1;
            *mask=128>>(d&7);
            return buffer+(size_t)(m+(d>>3)*3)*size_x;
        }
        else
        {   d=y/24,m=y%24;
            *inc=3;
            *mask=128>>(m&7);
            return buffer+3*((size_t)d*size_x)+(m>>3);
        }
}

/* Laserdrucker verzerrt quer zur Seite -> besser laengs der Seite spiegeln */

/*
static unsigned char reversal(register unsigned char ch)
{		register unsigned char ch2;
		register int i;

		ch2=0;
		for (i=0; i<8; i++)
		{	ch2<<=1;
			ch2|=ch&1;
			ch>>=1;
		}
		return ch2;
}

static void spiegel(register unsigned char *buffer, register int x)
{		register int i;
		register unsigned char ch;

		i=0; x--;
		while (x>i)
		{	ch=reversal(buffer[x]);
			buffer[x]=reversal(buffer[i]);
			buffer[i]=ch;
			i++, x--;
		}
		if (x==i)
			buffer[i]=reversal(buffer[i]);
}
*/

static void invert(register unsigned char *buffer, register int x)
{		while (x--)
			*buffer++^=0xff;
}

#ifdef UNIX
static int stricmp(char *s1,char *s2)
{		while(*s1 && tolower(*s1++)==tolower(*s2++));
		return tolower(*s1)-tolower(*s2);
}
#endif

MAIN
{       register FILE *fp;
        SIXTEENBIT x,size_x,size_y;
        int inc,mask;
        register int i,j,ch;
        register char *buffer,*p;

#ifdef UNIX
        fatal_name=*argv;
		printer=JET;
#else
        static char lp_name[LP_NAMELEN+1];

        fatal_name="laylp";

		if ((p=lpname(lp_name))==NULL) p="fx";
		if (!stricmp(p,"fx")) printer=FX;
		else if (!stricmp(p,"lq")) printer=LQ;
		else if (!stricmp(p,"jet")) printer=JET;
#endif

        OPT
            ARG 'q' : qflag=1;
            ARG 's' : sflag=1;
            ARG 'i' : iflag=1;
            ARG 'f' : printer=FX;
            ARG 'l' : printer=LQ;
            ARG 'j' : printer=JET;
            OTHER USAGE;
        ENDOPT

		if (printer==UNKNOWN) fatal("unsupported printer type %s",p);

        if (argc!=1) USAGE;

        if ((fp=fopen(*argv,"rb"))==NULL)
            fatal("can't open %s",*argv);

        if (fread(&size_x,sizeof(SIXTEENBIT),1,fp)!=1)
            fatal("can't read x-size");
        if (fread(&size_y,sizeof(SIXTEENBIT),1,fp)!=1)
            fatal("can't read y-size");
		x=size_x;
        size_x*=8;
        fprintf(stderr,"size %d x %d\n",size_x,size_y);

        if ((buffer=malloc(size_x*3))==NULL)
            fatal("no memory");

        init_prn();
        init_page();

		if (printer==JET)
		{	if (sflag && fseek(fp,(long)(size_y-1)*x,SEEK_CUR))
					fatal("seek error");
			for (i=0;i<size_y;i++)
			{	put_fstr("\033*b%hdW",x);
				if (fread(buffer,sizeof(char),x,fp)!=x)
					fatal("data read error");
				if (iflag) invert((unsigned char *)buffer,x);
				write_buf(buffer,x);
				if (sflag && i<size_y-1 && fseek(fp,(long)-2*x,SEEK_CUR))
					fatal("seek error");
			}
		}
		else
		{	ch=0;
			while (size_y)
			{   clear_buffer(buffer,size_x);
				for (j=0;j<24 && size_y;j++,size_y--)
				{   p=get_ptr(&inc,&mask,buffer,size_x,j);
					for (i=0;i<size_x;i++)
					{   if (i&7) ch<<=1;
						else if ((ch=getc(fp))==EOF)
							fatal("unexpected EOF");
						if (ch&128) *p|=mask;
						p+=inc;
					}
				}
				lp_buf(buffer,24,size_x);
			}
		}
        lp_page();
        flush_prn();
		return 0;
}
