/* (C) 2018 Tibor 'Igor2' Palinkas - license: GNU GPL 2+ (see ../COPYING) */
#include <stdlib.h>

#include "linea.h"
#include "tos.h"

VDIESC Vdiesc_;
FONT Font[2];
FONTS Fonts_ = {Font, Font};

FONTS *Fonts = &Fonts_;
VDIESC *Vdiesc = &Vdiesc_;

void linea_init(void)
{
	SDL_Init(SDL_INIT_VIDEO);
	Vdiesc->win = SDL_CreateWindow("router", SDL_WINDOWPOS_UNDEFINED, SDL_WINDOWPOS_UNDEFINED, 640, 400, /*SDL_WINDOW_INPUT_GRABBED*/0);
	Vdiesc->rend = SDL_CreateRenderer(Vdiesc->win, -1, 0 | ((getenv("SW_RENDER") != NULL) ? SDL_RENDERER_SOFTWARE : 0));
}

void *Physbase(void)
{
	return NULL;
}

void hide_mouse(void)
{

}

void Cursconf(int func, int rate)
{
}

#include "font.c"
#define FONT_WIDTH font[3]
#define FONT_HEIGHT 8
#define FONT_CHARMAP_START 4


static inline void font_draw_char_(int x0, int y0, const unsigned char *glyph, int px)
{
	int c, n, x, y;
	unsigned char tmp;

	for(n = 0, y = y0; n < FONT_HEIGHT; n++,y++) {
		tmp = *glyph;
		glyph++;
		for(c = 0, x = x0 + FONT_WIDTH; c < FONT_WIDTH; c++,x--) {
			if ((tmp & 1) == px)
				SDL_RenderDrawPoint(Vdiesc->rend, x, y);
			tmp >>= 1;
		}
	}
}

int font_draw_char(int x0, int y0, int chr)
{
	const unsigned char *glyph = &font[FONT_CHARMAP_START + chr*FONT_WIDTH];

	SDL_SetRenderDrawColor(Vdiesc->rend, 0, 0, 0, 255);
	font_draw_char_(x0, y0, glyph, 1);
	SDL_SetRenderDrawColor(Vdiesc->rend, 255, 255, 255, 255);
	font_draw_char_(x0, y0, glyph, 0);
	return FONT_WIDTH;
}

typedef struct {
	int sdlmod, sdlkey;
	unsigned int scan;
} key_trans_t;

static const key_trans_t key_trans[] = {
	/* scan-code based */
	{KMOD_NONE,  SDLK_UP,    0x0048},
	{KMOD_NONE,  SDLK_DOWN,  0x0050},
	{KMOD_NONE,  SDLK_LEFT,  0x004b},
	{KMOD_NONE,  SDLK_RIGHT, 0x004d},
	{KMOD_NONE,  SDLK_F1,    0x003b},
	{KMOD_SHIFT, SDLK_F1,    0x0054},
	{KMOD_NONE,  SDLK_F2,    0x003c},
	{KMOD_SHIFT, SDLK_F2,    0x0055},
	{KMOD_NONE,  SDLK_F3,    0x003d},
	{KMOD_NONE,  SDLK_F4,    0x003e},
	{KMOD_SHIFT, SDLK_F4,    0x0057},
	{KMOD_NONE,  SDLK_F5,    0x003f},
	{KMOD_SHIFT, SDLK_F5,    0x0058},
	{KMOD_NONE,  SDLK_F6,    0x0040},
	{KMOD_NONE,  SDLK_F7,    0x0041},
	{KMOD_NONE,  SDLK_F8,    0x0042},
	{KMOD_SHIFT, SDLK_F8,    0x005b},
	{KMOD_NONE,  SDLK_F9,    0x0043},
	{KMOD_SHIFT, SDLK_F9,    0x005c},
	{KMOD_NONE,  SDLK_F10,   0x0044},
	{0, 0, 0}
};

int wait_event(int timeout)
{
	SDL_Event ev;
	SDL_KeyboardEvent *kev = (SDL_KeyboardEvent *)&ev;
	SDL_MouseMotionEvent *mev = &ev.motion;
	int code;
	static const char *plain = "0123456789+-=,.";

	ev.type = 0;
	SDL_WaitEventTimeout(&ev, timeout);
	switch(ev.type) {
		case SDL_QUIT: exit(0);
		case SDL_MOUSEMOTION:
			Vdiesc->gcurx = mev->x;
			Vdiesc->gcury = mev->y;
		return 1;
		case SDL_KEYUP:
			code = 0;
			kev->keysym.mod &= (KMOD_CTRL | KMOD_SHIFT | KMOD_ALT);
			if ((kev->keysym.sym == SDLK_RETURN) || (kev->keysym.sym == SDLK_KP_ENTER)) {
				code = 0x0d;
			}
			else if (kev->keysym.sym == SDLK_KP_PLUS) {
				code = '+';
			}
			else if (kev->keysym.sym == SDLK_KP_MINUS) {
				code = '-';
			}
			else if (kev->keysym.sym == ';') {
				if (kev->keysym.mod & KMOD_SHIFT)
					code = ':';
				else
					code = ';';
			}
			else if ((kev->keysym.sym >= 'a') && (kev->keysym.sym <= 'z')) {
				if (kev->keysym.mod & KMOD_CTRL)
					code = kev->keysym.sym - 'a' + 1;
				else if (kev->keysym.mod & KMOD_SHIFT)
					code = kev->keysym.sym - 'a' + 'A';
				else
					code = kev->keysym.sym;
			}
			else if ((kev->keysym.sym >= '0') && (kev->keysym.sym <= '3') && (kev->keysym.mod & KMOD_SHIFT)) {
				static const char shift_num[] = "=!\"#"; /* almost German layout, had to hack # for portability */
				code = shift_num[kev->keysym.sym - '0'];
			}
			else if ((kev->keysym.sym < 127) && (kev->keysym.mod == 0) && (strchr(plain, kev->keysym.sym) != NULL)) {
				code = kev->keysym.sym;
			}
			else {
				const key_trans_t *t;
				for(t = key_trans; t->sdlkey != 0; t++) {
					if (((t->sdlmod & kev->keysym.mod) == kev->keysym.mod) && (t->sdlkey == kev->keysym.sym)) {
						code = t->scan << 16;
						break;
					}
				}
			}
			if (code != 0)
				kbd_push(code);

/*			printf("keysym=%d %c -> %x mod=%x\n", kev->keysym.sym, kev->keysym.sym, code, kev->keysym.mod);*/
			return 1;
	}
	return 0;
}


void idle_until_event(void)
{
	SDL_RenderPresent(Vdiesc->rend); /* flush anything happened in the previous action */
	for(;;) {
		if (wait_event(1000))
			return;
	}
}
